@@ -8,8 +8,8 @@ from . import turtle
 from . import snap
 from . import rooms
 
-from .common import get_location
+from .common import get_location, get_error, nothrow
 
-__version__ = '0.5.7'
+__version__ = '0.5.12'
 __author__ = 'Devin Jean'
 __credits__ = 'Institute for Software Integrated Systems, Vanderbilt University'
