@@ -12,6 +12,7 @@ import traceback
 import platform
 import copy
 import json
+import math
 import sys
 import io
 import re
@@ -41,6 +42,9 @@ NB_PROJECT_FILETYPES = [('NetsBlox Project Files', xux('.xml')), ('All Files', '
 PYTHON_FILETYPES = [('Python Files', xux('.py')), ('All Files', '.*')]
 IMAGE_FILETYPES = [('Images', xux('.png .jpg .jpeg')), ('All Files', '.*')]
 
+MIN_FONT_SIZE = 4
+MAX_FONT_SIZE = 40
+
 color_enabled = False
 try:
     # idle gives us syntax highlighting, but we don't require it otherwise
@@ -314,9 +318,11 @@ class Content(tk.Frame):
         self.pane = tk.PanedWindow(self, orient = tk.HORIZONTAL, **PANED_WINDOW_OPTS)
         self.pane.pack(fill = tk.BOTH, expand = True)
 
+        self.blocks = BlocksList(self.pane)
         self.project = ProjectEditor(self.pane)
         self.display = Display(self.pane)
 
+        self.pane.add(self.blocks, stretch = 'never')
         self.pane.add(self.project, stretch = 'always', width = 5, minsize = 300)
         self.pane.add(self.display, stretch = 'always', width = 3, minsize = 300)
 
@@ -354,7 +360,7 @@ class DndManager:
                 break
 
 class BlocksList(tk.Frame):
-    def __init__(self, parent, blocks, text_target):
+    def __init__(self, parent):
         super().__init__(parent)
 
         self.scrollbar = ttk.Scrollbar(self)
@@ -362,14 +368,20 @@ class BlocksList(tk.Frame):
         self.scrollbar.configure(command = self.text.yview)
 
         self.scrollbar.pack(side = tk.RIGHT, fill = tk.Y)
-        self.text.pack(side = tk.LEFT, fill = tk.Y, expand = True)
+        self.text.pack(side = tk.LEFT, fill = tk.BOTH, expand = True)
 
         # make sure user can't select anything with the mouse (would look weird)
         self.text.bind('<Button-1>', lambda e: 'break')
         self.text.bind('<B1-Motion>', lambda e: 'break')
         self.text.configure(cursor = 'arrow')
 
+        self.imgs = [] # for some reason we need to keep a reference to the images or they disappear
+
+        self.text.configure(state = tk.DISABLED)
+
+    def link(self, blocks, text_target) -> None:
         orig_bcolor = text_target.cget('background')
+
         def make_dnd_manager(widget, code):
             focused = [None]
             def on_start(e):
@@ -397,18 +409,23 @@ class BlocksList(tk.Frame):
 
             return DndManager(widget, [DndTarget(text_target, on_start, on_stop, on_drop)])
 
-        self.imgs = [] # for some reason we need to keep a reference to the images or they disappear
-        for block in blocks:
-            img = common.load_tkimage(block['url'], scale = block['scale'])
-            label = tk.Label(self.text, image = img, bg = COLOR_INFO['text-background-disabled'])
+        try:
+            self.text.config(state = tk.NORMAL)
 
-            self.text.window_create('end', window = label)
-            self.text.insert('end', '\n')
-            self.imgs.append(img)
+            self.text.delete('1.0', 'end')
+            self.imgs.clear()
 
-            make_dnd_manager(label, block['replace'])
+            for block in blocks:
+                img = common.load_tkimage(block['url'], scale = block['scale'])
+                label = tk.Label(self.text, image = img, bg = COLOR_INFO['text-background-disabled'])
 
-        self.text.configure(state = tk.DISABLED)
+                self.text.window_create('end', window = label)
+                self.text.insert('end', '\n')
+                self.imgs.append(img)
+
+                make_dnd_manager(label, block['replace'])
+        finally:
+            self.text.config(state = tk.DISABLED)
 
 class Imports:
     RAW_INFO = json.loads(common.load_text("netsblox://assets/default-imports.json"))
@@ -461,17 +478,18 @@ class ProjectEditor(tk.Frame):
     def __init__(self, parent):
         super().__init__(parent)
 
-        # this is randomly generated and is valid while the ide is open accross multiple program runs.
+        # this is randomly generated and is valid while the ide is open across multiple program runs.
         # it is not stored in project files or exports because conflicting ids would break messaging.
         self.project_id = common.generate_proj_id()
 
         self.roles: List[dict] = None
         self.active_role: int = None
 
+        self.client_type = 'editor' # must be one of: editor, dev
+
         self.block_sources = []
 
         self.editors: List[CodeEditor] = []
-        self.show_blocks_tkvar = tk.BooleanVar()
 
         def imports_update():
             for editor in self.editors:
@@ -528,18 +546,7 @@ class ProjectEditor(tk.Frame):
 
         editors = [x for x in self.editors if x.name == tab]
         assert len(editors) == 1
-        editors[0].on_content_change()
-
-    @property
-    def show_blocks(self) -> bool:
-        return self.show_blocks_tkvar.get()
-    @show_blocks.setter
-    def show_blocks(self, value: bool) -> None:
-        self.show_blocks_tkvar.set(value)
-        for editor in self.editors:
-            editor.update_pack()
-    def update_show_blocks(self):
-        self.show_blocks = self.show_blocks
+        editors[0].on_content_change(cause = 'tab-change')
 
     def delete_tab(self, idx) -> None:
         editor = self.editors[idx]
@@ -610,9 +617,11 @@ class ProjectEditor(tk.Frame):
     DEFAULT_PROJECT = json.loads(common.load_text('netsblox://assets/default-project.json'))
 
     def get_save_dict(self) -> dict:
-        res = {}
+        res = {
+            'client_type': self.client_type,
+            'roles': [],
+        }
 
-        res['roles'] = []
         for i, role in enumerate(self.roles):
             if i != self.active_role:
                 res['roles'].append(copy.deepcopy(role))
@@ -650,8 +659,6 @@ class ProjectEditor(tk.Frame):
 
             role_res['images'] = { name: common.encode_image(img) for name, img in self.imports.images.items() }
 
-        res['show_blocks'] = self.show_blocks
-
         return res
     def load(self, *, super_proj: Optional[dict] = None, active_role: Optional[int] = None) -> None:
         if super_proj is None:
@@ -660,6 +667,10 @@ class ProjectEditor(tk.Frame):
             super_proj = copy.deepcopy(super_proj)
             roles = super_proj.get('roles', [super_proj]) # default is for backward compat
 
+        client_type = super_proj.get('client_type', 'editor')
+        if client_type not in ['editor', 'dev']:
+            client_type = 'editor'
+
         if active_role is None:
             active_role = 0
         elif active_role < 0 or active_role >= len(roles):
@@ -694,6 +705,8 @@ class ProjectEditor(tk.Frame):
             'turtle': proj.get('turtle_blocks', []),
         }, source = None)
 
+        self.client_type = client_type
+
         self.roles = roles
         self.active_role = active_role
 
@@ -707,9 +720,6 @@ class ProjectEditor(tk.Frame):
             self.editors[i].destroy()
         self.editors = []
 
-        if super_proj is not None:
-            self.show_blocks = super_proj['show_blocks']
-
         for info in proj['editors']:
             ty = info['type']
             name = info['name']
@@ -735,6 +745,14 @@ class ProjectEditor(tk.Frame):
 
         self.imports.batch_update()
 
+        if len(self.editors) > 0:
+            default_tab_idx = len(self.editors) - 1 # default to stage (last tab) if no turtle editors
+            for i, editor in enumerate(self.editors):
+                if isinstance(editor, TurtleEditor):
+                    default_tab_idx = i
+                    break
+            self.notebook.select(default_tab_idx)
+
 class ContextMenu(tk.Menu):
     def __init__(self, parent, *, on_show = None):
         super().__init__(parent, tearoff = False)
@@ -753,8 +771,12 @@ class ContextMenu(tk.Menu):
             # theoretically these two _should be_ redundant, but they are needed in conjunction to work...
             if not self.visible:
                 self.visible = True
-                self.tk_popup(x, y) # witchcraft needed for FocusOut to work on linux
-            self.post(x, y)         # wizardry needed for unpost to work
+
+                # witchcraft needed for FocusOut to work on linux
+                # notably, if we do this on Darwin it'll result in the menu re-showing after close (which can cause a crash bug on tab deletion)
+                if platform.system() == 'Linux':
+                    self.tk_popup(x, y)
+            self.post(x, y) # wizardry needed for unpost to work
         finally:
             self.grab_release()
 
@@ -765,9 +787,8 @@ class ContextMenu(tk.Menu):
 
 # source: https://stackoverflow.com/questions/16369470/tkinter-adding-line-number-to-text-widget
 class TextLineNumbers(tk.Canvas):
-    def __init__(self, parent, *, target, width):
-        super().__init__(parent, width = width)
-        self.width = width
+    def __init__(self, parent, *, target):
+        super().__init__(parent)
         self.textwidget = target
         self.line_num_offset = 0
 
@@ -776,17 +797,22 @@ class TextLineNumbers(tk.Canvas):
     def redraw(self):
         self.delete('all')
 
+        line_info = [] # [(y: int, label: str)]
         i = self.textwidget.index('@0,0')
         while True:
             dline = self.textwidget.dlineinfo(i)
-            if dline is None:
-                break
-
-            y = dline[1]
+            if dline is None: break
             linenum = int(str(i).split('.')[0]) + self.line_num_offset
-            self.create_text(self.width - 2, y, anchor = 'ne', text = str(linenum), font = self.font)
+            line_info.append((dline[1], str(linenum)))
             i = self.textwidget.index(f'{i}+1line')
 
+        max_label_len = max(len(x[1]) for x in line_info)
+        w = math.ceil(6 + 0.8 * max_label_len * self.font.cget('size'))
+        self.config(width = w)
+
+        for y, label in line_info:
+            self.create_text(w - 2, y, anchor = 'ne', text = label, font = self.font)
+
 # source: https://stackoverflow.com/questions/16369470/tkinter-adding-line-number-to-text-widget
 class ChangedText(tk.Text):
     __name_id = 0
@@ -832,6 +858,7 @@ class ScrolledText(tk.Frame):
         undo_args = { 'undo': True, 'maxundo': -1, 'autoseparators': True }
 
         self.font = tkfont.nametofont('TkFixedFont')
+        self.blocks = blocks
 
         self.scrollbar = ttk.Scrollbar(self)
         self.text = ChangedText(self, font = self.font, yscrollcommand = self.scrollbar.set, wrap = tk.WORD if wrap else tk.NONE, **({} if readonly else undo_args), **kwargs)
@@ -865,7 +892,7 @@ class ScrolledText(tk.Frame):
         self.text.bind('<Shift-Home>', lambda e: on_home(do_select = True))
 
         self.linenumbers = None # default to none - conditionally created
-        self.blocks = None
+        self.blocks = blocks
 
         if readonly:
             # make text readonly be ignoring all (default) keystrokes
@@ -900,38 +927,33 @@ class ScrolledText(tk.Frame):
         self.text.bind(f'<{SYS_INFO["mod"]}-Key-C>', on_copy)
 
         if linenumbers:
-            self.linenumbers = TextLineNumbers(self, target = self.text, width = 40)
+            self.linenumbers = TextLineNumbers(self, target = self.text)
             self.text.bind('<<Change>>', lambda e: self.on_content_change())
             self.text.bind('<Configure>', lambda e: self.on_content_change())
 
-        if len(blocks) > 0:
-            self.blocks = BlocksList(self, blocks, self.text)
-
         # -----------------------------------------------------
 
-        self.update_pack()
-
-    def update_pack(self):
-        for item in [self.scrollbar, self.hscrollbar, self.blocks, self.linenumbers, self.text]:
-            if item is not None: item.pack_forget()
-
-        self.scrollbar.pack(side = tk.RIGHT, fill = tk.Y)
-        if self.blocks is not None and content is not None and content.project.show_blocks:
-            self.blocks.pack(side = tk.LEFT, fill = tk.Y)
+        col_off = 0
         if self.linenumbers is not None:
-            self.linenumbers.pack(side = tk.LEFT, fill = tk.Y)
+            self.linenumbers.grid(row = 0, column = 0, sticky = tk.NSEW)
+            col_off = 1
+        self.scrollbar.grid(row = 0, column = col_off + 1, sticky = tk.NSEW)
         if self.hscrollbar is not None:
-            self.hscrollbar.pack(side = tk.BOTTOM, fill = tk.X)
-        self.text.pack(side = tk.RIGHT, fill = tk.BOTH, expand = True)
+            self.hscrollbar.grid(row = 1, column = col_off, sticky = tk.NSEW)
+        self.text.grid(row = 0, column = col_off, sticky = tk.NSEW)
 
-        self.update() # needed on mac
+        self.columnconfigure(col_off, weight = 1)
+        self.rowconfigure(0, weight = 1)
 
-    def on_content_change(self):
+    def on_content_change(self, *, cause: str = 'unknown'):
         for handler in self.custom_on_change:
             handler()
         if self.linenumbers is not None:
             self.linenumbers.redraw()
 
+        if cause == 'tab-change':
+            content.blocks.link(self.blocks, self.text)
+
     def set_text(self, txt):
         self.text.delete('1.0', 'end')
         self.text.insert('1.0', txt)
@@ -1211,9 +1233,10 @@ class CodeEditor(ScrolledText):
 class GlobalEditor(CodeEditor):
     BASE_PREFIX = '''
 import netsblox
+from netsblox import get_location, get_error, nothrow
 from netsblox.turtle import *
 from netsblox.concurrency import *
-nb = netsblox.Client(proj_name = """$proj_name""", proj_id = $proj_id)
+nb = $client_type(proj_name = """$proj_name""", proj_id = $proj_id)
 'A connection to NetsBlox, which allows you to use services and RPCs from python.'
 netsblox.turtle._INITIAL_SIZE = $stage_size
 getattr(netsblox.turtle._get_proj_handle(), '_Project__tk').title(f'PyBlox - {nb.public_id}')
@@ -1226,7 +1249,7 @@ def _yield_(x):
     return x
 
 '''.lstrip()
-    BASE_PREFIX_LINES = 15
+    BASE_PREFIX_LINES = 16
 
     prefix = BASE_PREFIX
     prefix_lines = BASE_PREFIX_LINES
@@ -1238,7 +1261,13 @@ def _yield_(x):
         self.set_text(value)
 
     def get_script(self, *, is_export: bool = False):
+        client_type = {
+            'editor': 'netsblox.Client',
+            'dev': 'netsblox.dev.Client',
+        }[content.project.client_type]
+
         pre = GlobalEditor.prefix
+        pre = pre.replace('$client_type', client_type)
         pre = pre.replace('$proj_name', main_menu.project_name)
         pre = pre.replace('$proj_id', 'None' if is_export else f'\'{content.project.project_id}\'')
 
@@ -1287,13 +1316,21 @@ class Display(tk.Frame):
         self.pane = tk.PanedWindow(self, orient = tk.VERTICAL, **PANED_WINDOW_OPTS)
         self.pane.pack(fill = tk.BOTH, expand = True)
 
-        self.docs = ScrolledText(self.pane, readonly = True)
-        self.terminal = TerminalOutput(self.pane)
-
+        self.docs_frame = tk.Frame(self.pane)
+        self.docs_label = tk.Label(self.docs_frame, text = 'Documentation')
+        self.docs = ScrolledText(self.docs_frame, readonly = True)
         self.docs.text.configure(wrap = tk.WORD)
+        self.docs_label.pack(side = tk.TOP)
+        self.docs.pack(fill = tk.BOTH, expand = True)
 
-        self.pane.add(self.docs, stretch = 'always', minsize = 100)
-        self.pane.add(self.terminal, stretch = 'always', minsize = 100)
+        self.terminal_frame = tk.Frame(self.pane)
+        self.terminal_label = tk.Label(self.terminal_frame, text = 'Program Output')
+        self.terminal = TerminalOutput(self.terminal_frame)
+        self.terminal_label.pack(side = tk.TOP)
+        self.terminal.pack(fill = tk.BOTH, expand = True)
+
+        self.pane.add(self.docs_frame, stretch = 'always', minsize = 100)
+        self.pane.add(self.terminal_frame, stretch = 'always', minsize = 100)
 
 class TerminalOutput(tk.Frame):
     def __init__(self, parent):
@@ -1405,10 +1442,20 @@ class MainMenu(tk.Menu):
         root.bind_all(f'<{SYS_INFO["mod"]}-S>', lambda e: self.save_as())
 
         submenu = tk.Menu(self, **MENU_STYLE)
-        submenu.add_checkbutton(label = 'Blocks', variable = content.project.show_blocks_tkvar, command = content.project.update_show_blocks, accelerator = f'{SYS_INFO["mod-str"]}+B')
+        global_font = tkfont.nametofont('TkFixedFont')
+        def do_zoom(delta: int) -> None:
+            new_size = max(min(global_font.cget('size') + delta, MAX_FONT_SIZE), MIN_FONT_SIZE)
+            global_font.config(size = new_size)
+            for editor in content.project.editors:
+                editor.on_content_change(cause = 'zoom')
+            return 'break'
+        submenu.add_command(label = 'Zoom In', command = lambda: do_zoom(1), accelerator = f'{SYS_INFO["mod-str"]}++')
+        submenu.add_command(label = 'Zoom Out', command = lambda: do_zoom(-1), accelerator = f'{SYS_INFO["mod-str"]}+-')
         self.add_cascade(label = 'View', menu = submenu)
 
-        root.bind_all(f'<{SYS_INFO["mod"]}-b>', lambda e: self.toggle_blocks())
+        root.bind_all(f'<{SYS_INFO["mod"]}-plus>', lambda e: do_zoom(1))
+        root.bind_all(f'<{SYS_INFO["mod"]}-equal>', lambda e: do_zoom(1)) # plus without needing shift
+        root.bind_all(f'<{SYS_INFO["mod"]}-minus>', lambda e: do_zoom(-1))
 
         self.room_manager = rooms.EditorRoomManager(client = nb)
 
@@ -1469,7 +1516,7 @@ class MainMenu(tk.Menu):
     def project_path(self, p):
         self._project_path = p
         self._project_name = 'untitled' if p is None else basename_noext(p)
-        root.title(f'NetsBlox-Python - {self.public_id} ({"unsaved" if p is None else p})')
+        root.title(f'PyBlox - {self.public_id} ({"unsaved" if p is None else p})')
     @property
     def project_name(self):
         return self._project_name
@@ -1567,9 +1614,6 @@ class MainMenu(tk.Menu):
         res = messagebox.askyesnocancel(title, msg)
         return res == False or (res == True and self.save(save_dict))
 
-    def toggle_blocks(self):
-        content.project.show_blocks = not content.project.show_blocks
-
     def import_image(self):
         p = filedialog.askopenfilename(filetypes = IMAGE_FILETYPES)
         if type(p) is not str or not p:
@@ -1705,7 +1749,7 @@ class MainMenu(tk.Menu):
 
             def get_switcher(idx):
                 def do_switch():
-                    if idx == content.project.active_role: return # santy check
+                    if idx == content.project.active_role: return # sanity check
                     self.switch_role(active_role = idx)
                 return do_switch
             submenu.add_command(label = 'Open', command = get_switcher(i), state = tk.DISABLED if is_current else tk.ACTIVE)
